#include "gtest/gtest.h"

#include "definitions.h"


/*
 * The internal implementation from queue.c BEFORE the breaking change.
 */
bool old_xQueueGenericCreate( const UBaseType_t uxQueueLength,
                                   const UBaseType_t uxItemSize,
                                   const uint8_t ucQueueType )
{
    if( ( uxQueueLength > ( UBaseType_t ) 0 ) &&
        /* Check for multiplication overflow. */
        ( ( SIZE_MAX / uxQueueLength ) >= uxItemSize ) &&
        /* Check for addition overflow. */
        ( ( SIZE_MAX - sizeof( Queue_t ) ) >= ( uxQueueLength * uxItemSize ) ) )
    {
        // Internal logic has been omitted
        return true;
    }
    else
    {
        return false;
    }
}

/*
 * The internal implementation from queue.c AFTER the breaking change.
 */
bool new_xQueueGenericCreate( const UBaseType_t uxQueueLength,
                                   const UBaseType_t uxItemSize,
                                   const uint8_t ucQueueType )
{
    if( ( uxQueueLength > ( UBaseType_t ) 0 ) &&
        /* Check for multiplication overflow. */
        ( ( SIZE_MAX / uxQueueLength ) >= uxItemSize ) &&
        /* Check for addition overflow. */
        ( ( UBaseType_t ) ( SIZE_MAX - sizeof( Queue_t ) ) >= ( uxQueueLength * uxItemSize ) ) )
    {
        // Internal logic has been omitted
        return true;
    }
    else
    {
        return false;
    }
}





using ::testing::Test;

TEST_F(Test, small_queue) {
    /* GIVEN */
    int uxQueueLength = 10;
    int uxItemSize = 2;
    
    bool result_old = old_xQueueGenericCreate(uxQueueLength, uxItemSize, queueQUEUE_TYPE_BASE);
    ASSERT_TRUE(result_old);

    /* WHEN */
    bool result_new = new_xQueueGenericCreate(uxQueueLength, uxItemSize, queueQUEUE_TYPE_BASE);

    /* THEN */
    ASSERT_TRUE(result_new);
}

TEST_F(Test, big_queue) {
    /* GIVEN */
    int uxQueueLength = 64;
    int uxItemSize = 4;

    bool result_old = old_xQueueGenericCreate(uxQueueLength, uxItemSize, queueQUEUE_TYPE_BASE);
    ASSERT_TRUE(result_old);

    /* WHEN */
    bool result_new = new_xQueueGenericCreate(uxQueueLength, uxItemSize, queueQUEUE_TYPE_BASE);

    /* THEN */
    ASSERT_TRUE(result_new);
}

TEST_F(Test, bigger_queue) {
    /* GIVEN */
    int uxQueueLength = 65;
    int uxItemSize = 4;

    bool result_old = old_xQueueGenericCreate(uxQueueLength, uxItemSize, queueQUEUE_TYPE_BASE);
    ASSERT_TRUE(result_old);

    /* WHEN */
    bool result_new = new_xQueueGenericCreate(uxQueueLength, uxItemSize, queueQUEUE_TYPE_BASE);

    /* THEN */
    ASSERT_TRUE(result_new);
}

TEST_F(Test, max_queue) {
    /* GIVEN */
    int uxQueueLength = 16353;
    int uxItemSize = 4;

    uint32_t max = SIZE_MAX;
    uint32_t sizeQueue = sizeof(Queue_t);
    uint32_t sizeBuffer = uxQueueLength * uxItemSize;
    uint32_t total = sizeBuffer + sizeQueue;
    ASSERT_LE(total, max);

    bool result_old = old_xQueueGenericCreate(uxQueueLength, uxItemSize, queueQUEUE_TYPE_BASE);
    ASSERT_TRUE(result_old);

    /* WHEN */
    bool result_new = new_xQueueGenericCreate(uxQueueLength, uxItemSize, queueQUEUE_TYPE_BASE);

    /* THEN */
    ASSERT_TRUE(result_new);
}


TEST_F(Test, max_sucess_queue) {
    /* GIVEN */
    int uxQueueLength = 33; // <-- that's the max you can get, 34 will fail
    int uxItemSize = 4;

    bool result_old = old_xQueueGenericCreate(uxQueueLength, uxItemSize, queueQUEUE_TYPE_BASE);
    ASSERT_TRUE(result_old);

    /* WHEN */
    bool result_new = new_xQueueGenericCreate(uxQueueLength, uxItemSize, queueQUEUE_TYPE_BASE);

    /* THEN */
    ASSERT_TRUE(result_new);
}
